#!/usr/bin/php
<?php
/** Geo GPX cleaner
 * 
 * Renders clean GPX for TrekBuddy from the geocaching site or geotoad tool.
 * 
 * Copyright 2009  Pavel Zbytovsky  <mail at zby.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 */

// --------------  argument variables ---------------
$file = "";
$tidyxml = false;
$lognum = -1; 
$fixascii = false;
define("VERSION", "1.1 (2009/03/14)");
if($_GET["html"]){
  echo "<pre>";
}


// ------------------ argument parsing --------------------
$args = $_SERVER["argv"];
for($i=1; $i<count($args); $i++){
	if($args[$i] == "--tidyxml")
		$tidyxml = true;
	elseif($args[$i] == "--lognum")
		$lognum = $args[++$i];
	elseif($args[$i] == "--fixascii")
		$fixascii = true;
	elseif(!$file)
		$file = $args[$i];
	else{
    help("Unknown argument '$args[$i]'.");
	}
}
if($_GET["file"]) $file = strtr($_GET["file"], "\\/", "--");
if($_GET["tidyxml"]) $tidyxml = $_GET["tidyxml"];
if(isset($_GET["lognum"])) $lognum = $_GET["lognum"]=="" ? -1 : $_GET["lognum"];
if($_GET["fixascii"]) $fixascii = $_GET["fixascii"];


// ------------------ argument checking --------------------
if(!$file)
  help();

if(!is_numeric($lognum)) 
  help("Log num is not number '$lognum'.");

if(!file_exists($file))
  help("File '$file' does not exist.");



// ------------------- START THE PARSER -------------------
set_error_handler("myErrorHandler");
error_reporting(E_ALL & ~E_NOTICE);
$doc = new DOMDocument();
if($tidyxml) $doc->formatOutput = true;
$startsize = filesize($file);
echo "\nLoading xml '$file', {$startsize}B\n";



// ------------------- remove ASCII<32 -------------------
if($fixascii){
  $arr = array();
  for($i=1; $i<32; $i++) $arr[$i] = chr($i);
  unset($arr[13], $arr[10]); //CR, LF
  
  $doc->loadXML(str_replace($arr, "", file_get_contents($file)), LIBXML_NOBLANKS); //its not slow, just memory demanding :)
}
else {
  $doc->load($file, LIBXML_NOBLANKS);
}



// ------------------- find <wpt>s -------------------
$dataset = $doc->getElementsByTagName("wpt");
echo "\nFound $dataset->length [wpts]s\nprocessing...";

foreach( $dataset as $wpt )
{
    //remove childs of <wpt>
    foreach(array('time', 'desc', 'url', 'urlname', 'sym', 'type') as $tag)
      if(nodebytag($wpt, $tag))
        $wpt->removeChild(nodebytag($wpt, $tag));
    
    //remove atributes in <groundspeak:cache>
    $cache = nodebytag($wpt, 'cache');
    if(!$cache) continue;
    $cache->removeAttribute('id');
    $cache->removeAttribute('available');
    $cache->removeAttribute('archived');
    $cache->removeAttributeNS('http://www.groundspeak.com/cache/1/0','groundspeak');
    
    //prefix the name with type of cache
    $typ = nodebytag($cache, 'type')->nodeValue;
    if($typ != 'traditional'){
      if($typ == 'unknown') $typ = "?";
      elseif($typ == 'multicache') $typ = "×";
      else $typ = substr($typ, 0,1) . ":";
      
      nodebytag($cache, 'name')->insertBefore($doc->createTextNode($typ), nodebytag($cache, 'name')->firstChild);
    }
    
    //remove unneeded childs of <cache>
    foreach(array('date', 'full_name', 'placed_by', 'owner', 'state', 'country', 'travelbugs') as $tag)
      if(nodebytag($cache, $tag))
        $cache->removeChild(nodebytag($cache, $tag));
    
    //append parsed logs into <long_description> and remove <logs>
    if(nodebytag($cache, 'logs')){
      if($lognum != 0){
        $i=0;
        $textlogs = "\n------\n"; 
        foreach($cache->getElementsByTagName('log') as $log){
          $textlogs .= sprintf("%s %s\n%s ~%s\n\n",
            substr(nodebytag($log, 'date')->nodeValue, 0,10),
            substr(nodebytag($log, 'type')->nodeValue, 0,5),
            nodebytag($log, 'text')->nodeValue,
            nodebytag($log, 'finder')->nodeValue
            );
          if(++$i == $lognum) break;
        }
        nodebytag($cache, 'long_description')
            ->appendChild($doc->createTextNode($textlogs));
      }
      $cache->removeChild(nodebytag($cache, 'logs'));
    }
}


$size = $doc->save($save = substr($file,0,-4) . '.TB.gpx');
printf("\nsaved %s ({$size}B ".round($size/$startsize*100)."%%).", $_GET["html"] ? "<a href='$save'>$save</a>" : $save);



function nodebytag($o, $s){
  return $o->getElementsByTagName($s)->item(0);
}



function help($err=false){
if($err) echo "\n           (!) $err\n\n\n";
$self = basename(__FILE__);
die(
"Usage: $self [OPTIONS] file.gpx
  Web: $self?file=file.gpx[&tidyxml=1][&...]

Simple PHP script for cleaning GPX geocaching files from unwanted items,
especially those not needed for J2ME map application TrekBuddy.net 

 --tidyxml   Adds some whitespaces to result xml - more human readable
 --lognum #  Max logs parsed and added to LongDesc, default: all in GPX
 --fixascii  If an error saying 'PCDATA invalid Char' occures, you may try this.
  
GeoGPX Cleaner, v".VERSION.", http://www.zby.cz/prg/geogpxcleaner
");
} 


function myErrorHandler($errno, $errstr, $errfile, $errline){
  $s = 'DOMDocument::load[^]]+]: (.+)$';
  if(ereg($s, $errstr) ){
    echo "[XML_ERROR] " . ereg_replace($s, '\\1', $errstr) ."\n";
    return true;
  }
  return false;
}

